/****************************************************************************
** Meta object code from reading C++ file 'inputcontext.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.9.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/qtvirtualkeyboard-opensource-src-5.9.4/src/virtualkeyboard/inputcontext.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'inputcontext.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.9.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QtVirtualKeyboard__InputContext_t {
    QByteArrayData data[75];
    char stringdata0[1266];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QtVirtualKeyboard__InputContext_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QtVirtualKeyboard__InputContext_t qt_meta_stringdata_QtVirtualKeyboard__InputContext = {
    {
QT_MOC_LITERAL(0, 0, 31), // "QtVirtualKeyboard::InputContext"
QT_MOC_LITERAL(1, 32, 12), // "focusChanged"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 18), // "focusEditorChanged"
QT_MOC_LITERAL(4, 65, 18), // "preeditTextChanged"
QT_MOC_LITERAL(5, 84, 23), // "inputMethodHintsChanged"
QT_MOC_LITERAL(6, 108, 22), // "surroundingTextChanged"
QT_MOC_LITERAL(7, 131, 19), // "selectedTextChanged"
QT_MOC_LITERAL(8, 151, 21), // "anchorPositionChanged"
QT_MOC_LITERAL(9, 173, 21), // "cursorPositionChanged"
QT_MOC_LITERAL(10, 195, 22), // "anchorRectangleChanged"
QT_MOC_LITERAL(11, 218, 22), // "cursorRectangleChanged"
QT_MOC_LITERAL(12, 241, 12), // "shiftChanged"
QT_MOC_LITERAL(13, 254, 15), // "capsLockChanged"
QT_MOC_LITERAL(14, 270, 16), // "uppercaseChanged"
QT_MOC_LITERAL(15, 287, 24), // "keyboardRectangleChanged"
QT_MOC_LITERAL(16, 312, 23), // "previewRectangleChanged"
QT_MOC_LITERAL(17, 336, 21), // "previewVisibleChanged"
QT_MOC_LITERAL(18, 358, 16), // "animatingChanged"
QT_MOC_LITERAL(19, 375, 13), // "localeChanged"
QT_MOC_LITERAL(20, 389, 16), // "inputItemChanged"
QT_MOC_LITERAL(21, 406, 30), // "selectionControlVisibleChanged"
QT_MOC_LITERAL(22, 437, 20), // "navigationKeyPressed"
QT_MOC_LITERAL(23, 458, 3), // "key"
QT_MOC_LITERAL(24, 462, 12), // "isAutoRepeat"
QT_MOC_LITERAL(25, 475, 21), // "navigationKeyReleased"
QT_MOC_LITERAL(26, 497, 35), // "anchorRectIntersectsClipRectC..."
QT_MOC_LITERAL(27, 533, 35), // "cursorRectIntersectsClipRectC..."
QT_MOC_LITERAL(28, 569, 18), // "onInputItemChanged"
QT_MOC_LITERAL(29, 588, 22), // "updateAvailableLocales"
QT_MOC_LITERAL(30, 611, 16), // "availableLocales"
QT_MOC_LITERAL(31, 628, 14), // "hideInputPanel"
QT_MOC_LITERAL(32, 643, 12), // "sendKeyClick"
QT_MOC_LITERAL(33, 656, 4), // "text"
QT_MOC_LITERAL(34, 661, 9), // "modifiers"
QT_MOC_LITERAL(35, 671, 6), // "commit"
QT_MOC_LITERAL(36, 678, 11), // "replaceFrom"
QT_MOC_LITERAL(37, 690, 13), // "replaceLength"
QT_MOC_LITERAL(38, 704, 5), // "clear"
QT_MOC_LITERAL(39, 710, 10), // "fileExists"
QT_MOC_LITERAL(40, 721, 7), // "fileUrl"
QT_MOC_LITERAL(41, 729, 17), // "hasEnterKeyAction"
QT_MOC_LITERAL(42, 747, 4), // "item"
QT_MOC_LITERAL(43, 752, 25), // "setSelectionOnFocusObject"
QT_MOC_LITERAL(44, 778, 9), // "anchorPos"
QT_MOC_LITERAL(45, 788, 9), // "cursorPos"
QT_MOC_LITERAL(46, 798, 19), // "forceCursorPosition"
QT_MOC_LITERAL(47, 818, 14), // "anchorPosition"
QT_MOC_LITERAL(48, 833, 14), // "cursorPosition"
QT_MOC_LITERAL(49, 848, 5), // "focus"
QT_MOC_LITERAL(50, 854, 5), // "shift"
QT_MOC_LITERAL(51, 860, 8), // "capsLock"
QT_MOC_LITERAL(52, 869, 9), // "uppercase"
QT_MOC_LITERAL(53, 879, 16), // "inputMethodHints"
QT_MOC_LITERAL(54, 896, 20), // "Qt::InputMethodHints"
QT_MOC_LITERAL(55, 917, 11), // "preeditText"
QT_MOC_LITERAL(56, 929, 15), // "surroundingText"
QT_MOC_LITERAL(57, 945, 12), // "selectedText"
QT_MOC_LITERAL(58, 958, 15), // "anchorRectangle"
QT_MOC_LITERAL(59, 974, 15), // "cursorRectangle"
QT_MOC_LITERAL(60, 990, 17), // "keyboardRectangle"
QT_MOC_LITERAL(61, 1008, 16), // "previewRectangle"
QT_MOC_LITERAL(62, 1025, 14), // "previewVisible"
QT_MOC_LITERAL(63, 1040, 9), // "animating"
QT_MOC_LITERAL(64, 1050, 6), // "locale"
QT_MOC_LITERAL(65, 1057, 9), // "inputItem"
QT_MOC_LITERAL(66, 1067, 12), // "shiftHandler"
QT_MOC_LITERAL(67, 1080, 32), // "QtVirtualKeyboard::ShiftHandler*"
QT_MOC_LITERAL(68, 1113, 11), // "inputEngine"
QT_MOC_LITERAL(69, 1125, 31), // "QtVirtualKeyboard::InputEngine*"
QT_MOC_LITERAL(70, 1157, 23), // "selectionControlVisible"
QT_MOC_LITERAL(71, 1181, 28), // "anchorRectIntersectsClipRect"
QT_MOC_LITERAL(72, 1210, 28), // "cursorRectIntersectsClipRect"
QT_MOC_LITERAL(73, 1239, 6), // "shadow"
QT_MOC_LITERAL(74, 1246, 19) // "ShadowInputContext*"

    },
    "QtVirtualKeyboard::InputContext\0"
    "focusChanged\0\0focusEditorChanged\0"
    "preeditTextChanged\0inputMethodHintsChanged\0"
    "surroundingTextChanged\0selectedTextChanged\0"
    "anchorPositionChanged\0cursorPositionChanged\0"
    "anchorRectangleChanged\0cursorRectangleChanged\0"
    "shiftChanged\0capsLockChanged\0"
    "uppercaseChanged\0keyboardRectangleChanged\0"
    "previewRectangleChanged\0previewVisibleChanged\0"
    "animatingChanged\0localeChanged\0"
    "inputItemChanged\0selectionControlVisibleChanged\0"
    "navigationKeyPressed\0key\0isAutoRepeat\0"
    "navigationKeyReleased\0"
    "anchorRectIntersectsClipRectChanged\0"
    "cursorRectIntersectsClipRectChanged\0"
    "onInputItemChanged\0updateAvailableLocales\0"
    "availableLocales\0hideInputPanel\0"
    "sendKeyClick\0text\0modifiers\0commit\0"
    "replaceFrom\0replaceLength\0clear\0"
    "fileExists\0fileUrl\0hasEnterKeyAction\0"
    "item\0setSelectionOnFocusObject\0anchorPos\0"
    "cursorPos\0forceCursorPosition\0"
    "anchorPosition\0cursorPosition\0focus\0"
    "shift\0capsLock\0uppercase\0inputMethodHints\0"
    "Qt::InputMethodHints\0preeditText\0"
    "surroundingText\0selectedText\0"
    "anchorRectangle\0cursorRectangle\0"
    "keyboardRectangle\0previewRectangle\0"
    "previewVisible\0animating\0locale\0"
    "inputItem\0shiftHandler\0"
    "QtVirtualKeyboard::ShiftHandler*\0"
    "inputEngine\0QtVirtualKeyboard::InputEngine*\0"
    "selectionControlVisible\0"
    "anchorRectIntersectsClipRect\0"
    "cursorRectIntersectsClipRect\0shadow\0"
    "ShadowInputContext*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QtVirtualKeyboard__InputContext[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      38,   14, // methods
      24,  286, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      24,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  204,    2, 0x06 /* Public */,
       3,    0,  205,    2, 0x06 /* Public */,
       4,    0,  206,    2, 0x06 /* Public */,
       5,    0,  207,    2, 0x06 /* Public */,
       6,    0,  208,    2, 0x06 /* Public */,
       7,    0,  209,    2, 0x06 /* Public */,
       8,    0,  210,    2, 0x06 /* Public */,
       9,    0,  211,    2, 0x06 /* Public */,
      10,    0,  212,    2, 0x06 /* Public */,
      11,    0,  213,    2, 0x06 /* Public */,
      12,    0,  214,    2, 0x06 /* Public */,
      13,    0,  215,    2, 0x06 /* Public */,
      14,    0,  216,    2, 0x06 /* Public */,
      15,    0,  217,    2, 0x06 /* Public */,
      16,    0,  218,    2, 0x06 /* Public */,
      17,    0,  219,    2, 0x06 /* Public */,
      18,    0,  220,    2, 0x06 /* Public */,
      19,    0,  221,    2, 0x06 /* Public */,
      20,    0,  222,    2, 0x06 /* Public */,
      21,    0,  223,    2, 0x06 /* Public */,
      22,    2,  224,    2, 0x06 /* Public */,
      25,    2,  229,    2, 0x06 /* Public */,
      26,    0,  234,    2, 0x06 /* Public */,
      27,    0,  235,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      28,    0,  236,    2, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      29,    1,  237,    2, 0x02 /* Public */,
      31,    0,  240,    2, 0x02 /* Public */,
      32,    3,  241,    2, 0x02 /* Public */,
      32,    2,  248,    2, 0x22 /* Public | MethodCloned */,
      35,    0,  253,    2, 0x02 /* Public */,
      35,    3,  254,    2, 0x02 /* Public */,
      35,    2,  261,    2, 0x22 /* Public | MethodCloned */,
      35,    1,  266,    2, 0x22 /* Public | MethodCloned */,
      38,    0,  269,    2, 0x02 /* Public */,
      39,    1,  270,    2, 0x02 /* Public */,
      41,    1,  273,    2, 0x02 /* Public */,
      43,    2,  276,    2, 0x02 /* Public */,
      46,    2,  281,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,   23,   24,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,   23,   24,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // methods: parameters
    QMetaType::Void, QMetaType::QStringList,   30,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QString, QMetaType::Int,   23,   33,   34,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   23,   33,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::Int,   33,   36,   37,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,   33,   36,
    QMetaType::Void, QMetaType::QString,   33,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QUrl,   40,
    QMetaType::Bool, QMetaType::QObjectStar,   42,
    QMetaType::Void, QMetaType::QPointF, QMetaType::QPointF,   44,   45,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   47,   48,

 // properties: name, type, flags
      49, QMetaType::Bool, 0x00495001,
      50, QMetaType::Bool, 0x00495103,
      51, QMetaType::Bool, 0x00495103,
      52, QMetaType::Bool, 0x00495001,
      47, QMetaType::Int, 0x00495001,
      48, QMetaType::Int, 0x00495001,
      53, 0x80000000 | 54, 0x00495009,
      55, QMetaType::QString, 0x00495103,
      56, QMetaType::QString, 0x00495001,
      57, QMetaType::QString, 0x00495001,
      58, QMetaType::QRectF, 0x00495001,
      59, QMetaType::QRectF, 0x00495001,
      60, QMetaType::QRectF, 0x00495103,
      61, QMetaType::QRectF, 0x00495103,
      62, QMetaType::Bool, 0x00495103,
      63, QMetaType::Bool, 0x00495103,
      64, QMetaType::QString, 0x00495103,
      65, QMetaType::QObjectStar, 0x00495001,
      66, 0x80000000 | 67, 0x00095409,
      68, 0x80000000 | 69, 0x00095409,
      70, QMetaType::Bool, 0x00495001,
      71, QMetaType::Bool, 0x00495001,
      72, QMetaType::Bool, 0x00495001,
      73, 0x80000000 | 74, 0x00095409,

 // properties: notify_signal_id
       0,
      10,
      11,
      12,
       6,
       7,
       3,
       2,
       4,
       5,
       8,
       9,
      13,
      14,
      15,
      16,
      17,
      18,
       0,
       0,
      19,
      22,
      23,
       0,

       0        // eod
};

void QtVirtualKeyboard::InputContext::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        InputContext *_t = static_cast<InputContext *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->focusChanged(); break;
        case 1: _t->focusEditorChanged(); break;
        case 2: _t->preeditTextChanged(); break;
        case 3: _t->inputMethodHintsChanged(); break;
        case 4: _t->surroundingTextChanged(); break;
        case 5: _t->selectedTextChanged(); break;
        case 6: _t->anchorPositionChanged(); break;
        case 7: _t->cursorPositionChanged(); break;
        case 8: _t->anchorRectangleChanged(); break;
        case 9: _t->cursorRectangleChanged(); break;
        case 10: _t->shiftChanged(); break;
        case 11: _t->capsLockChanged(); break;
        case 12: _t->uppercaseChanged(); break;
        case 13: _t->keyboardRectangleChanged(); break;
        case 14: _t->previewRectangleChanged(); break;
        case 15: _t->previewVisibleChanged(); break;
        case 16: _t->animatingChanged(); break;
        case 17: _t->localeChanged(); break;
        case 18: _t->inputItemChanged(); break;
        case 19: _t->selectionControlVisibleChanged(); break;
        case 20: _t->navigationKeyPressed((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 21: _t->navigationKeyReleased((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 22: _t->anchorRectIntersectsClipRectChanged(); break;
        case 23: _t->cursorRectIntersectsClipRectChanged(); break;
        case 24: _t->onInputItemChanged(); break;
        case 25: _t->updateAvailableLocales((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 26: _t->hideInputPanel(); break;
        case 27: _t->sendKeyClick((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 28: _t->sendKeyClick((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 29: _t->commit(); break;
        case 30: _t->commit((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 31: _t->commit((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 32: _t->commit((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 33: _t->clear(); break;
        case 34: { bool _r = _t->fileExists((*reinterpret_cast< const QUrl(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 35: { bool _r = _t->hasEnterKeyAction((*reinterpret_cast< QObject*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 36: _t->setSelectionOnFocusObject((*reinterpret_cast< const QPointF(*)>(_a[1])),(*reinterpret_cast< const QPointF(*)>(_a[2]))); break;
        case 37: _t->forceCursorPosition((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::focusChanged)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::focusEditorChanged)) {
                *result = 1;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::preeditTextChanged)) {
                *result = 2;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::inputMethodHintsChanged)) {
                *result = 3;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::surroundingTextChanged)) {
                *result = 4;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::selectedTextChanged)) {
                *result = 5;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::anchorPositionChanged)) {
                *result = 6;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::cursorPositionChanged)) {
                *result = 7;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::anchorRectangleChanged)) {
                *result = 8;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::cursorRectangleChanged)) {
                *result = 9;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::shiftChanged)) {
                *result = 10;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::capsLockChanged)) {
                *result = 11;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::uppercaseChanged)) {
                *result = 12;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::keyboardRectangleChanged)) {
                *result = 13;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::previewRectangleChanged)) {
                *result = 14;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::previewVisibleChanged)) {
                *result = 15;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::animatingChanged)) {
                *result = 16;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::localeChanged)) {
                *result = 17;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::inputItemChanged)) {
                *result = 18;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::selectionControlVisibleChanged)) {
                *result = 19;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)(int , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::navigationKeyPressed)) {
                *result = 20;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)(int , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::navigationKeyReleased)) {
                *result = 21;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::anchorRectIntersectsClipRectChanged)) {
                *result = 22;
                return;
            }
        }
        {
            typedef void (InputContext::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputContext::cursorRectIntersectsClipRectChanged)) {
                *result = 23;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        InputContext *_t = static_cast<InputContext *>(_o);
        Q_UNUSED(_t)
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->focus(); break;
        case 1: *reinterpret_cast< bool*>(_v) = _t->shift(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->capsLock(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->uppercase(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->anchorPosition(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->cursorPosition(); break;
        case 6: *reinterpret_cast< Qt::InputMethodHints*>(_v) = _t->inputMethodHints(); break;
        case 7: *reinterpret_cast< QString*>(_v) = _t->preeditText(); break;
        case 8: *reinterpret_cast< QString*>(_v) = _t->surroundingText(); break;
        case 9: *reinterpret_cast< QString*>(_v) = _t->selectedText(); break;
        case 10: *reinterpret_cast< QRectF*>(_v) = _t->anchorRectangle(); break;
        case 11: *reinterpret_cast< QRectF*>(_v) = _t->cursorRectangle(); break;
        case 12: *reinterpret_cast< QRectF*>(_v) = _t->keyboardRectangle(); break;
        case 13: *reinterpret_cast< QRectF*>(_v) = _t->previewRectangle(); break;
        case 14: *reinterpret_cast< bool*>(_v) = _t->previewVisible(); break;
        case 15: *reinterpret_cast< bool*>(_v) = _t->animating(); break;
        case 16: *reinterpret_cast< QString*>(_v) = _t->locale(); break;
        case 17: *reinterpret_cast< QObject**>(_v) = _t->inputItem(); break;
        case 18: *reinterpret_cast< QtVirtualKeyboard::ShiftHandler**>(_v) = _t->shiftHandler(); break;
        case 19: *reinterpret_cast< QtVirtualKeyboard::InputEngine**>(_v) = _t->inputEngine(); break;
        case 20: *reinterpret_cast< bool*>(_v) = _t->selectionControlVisible(); break;
        case 21: *reinterpret_cast< bool*>(_v) = _t->anchorRectIntersectsClipRect(); break;
        case 22: *reinterpret_cast< bool*>(_v) = _t->cursorRectIntersectsClipRect(); break;
        case 23: *reinterpret_cast< ShadowInputContext**>(_v) = _t->shadow(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        InputContext *_t = static_cast<InputContext *>(_o);
        Q_UNUSED(_t)
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setShift(*reinterpret_cast< bool*>(_v)); break;
        case 2: _t->setCapsLock(*reinterpret_cast< bool*>(_v)); break;
        case 7: _t->setPreeditText(*reinterpret_cast< QString*>(_v)); break;
        case 12: _t->setKeyboardRectangle(*reinterpret_cast< QRectF*>(_v)); break;
        case 13: _t->setPreviewRectangle(*reinterpret_cast< QRectF*>(_v)); break;
        case 14: _t->setPreviewVisible(*reinterpret_cast< bool*>(_v)); break;
        case 15: _t->setAnimating(*reinterpret_cast< bool*>(_v)); break;
        case 16: _t->setLocale(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

const QMetaObject QtVirtualKeyboard::InputContext::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_QtVirtualKeyboard__InputContext.data,
      qt_meta_data_QtVirtualKeyboard__InputContext,  qt_static_metacall, nullptr, nullptr}
};


const QMetaObject *QtVirtualKeyboard::InputContext::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtVirtualKeyboard::InputContext::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QtVirtualKeyboard__InputContext.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtVirtualKeyboard::InputContext::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 38)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 38;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 38)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 38;
    }
#ifndef QT_NO_PROPERTIES
   else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 24;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 24;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 24;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 24;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 24;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QtVirtualKeyboard::InputContext::focusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QtVirtualKeyboard::InputContext::focusEditorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QtVirtualKeyboard::InputContext::preeditTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QtVirtualKeyboard::InputContext::inputMethodHintsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QtVirtualKeyboard::InputContext::surroundingTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QtVirtualKeyboard::InputContext::selectedTextChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QtVirtualKeyboard::InputContext::anchorPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QtVirtualKeyboard::InputContext::cursorPositionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void QtVirtualKeyboard::InputContext::anchorRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QtVirtualKeyboard::InputContext::cursorRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QtVirtualKeyboard::InputContext::shiftChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void QtVirtualKeyboard::InputContext::capsLockChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void QtVirtualKeyboard::InputContext::uppercaseChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void QtVirtualKeyboard::InputContext::keyboardRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void QtVirtualKeyboard::InputContext::previewRectangleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void QtVirtualKeyboard::InputContext::previewVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void QtVirtualKeyboard::InputContext::animatingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void QtVirtualKeyboard::InputContext::localeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void QtVirtualKeyboard::InputContext::inputItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void QtVirtualKeyboard::InputContext::selectionControlVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void QtVirtualKeyboard::InputContext::navigationKeyPressed(int _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 20, _a);
}

// SIGNAL 21
void QtVirtualKeyboard::InputContext::navigationKeyReleased(int _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}

// SIGNAL 22
void QtVirtualKeyboard::InputContext::anchorRectIntersectsClipRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void QtVirtualKeyboard::InputContext::cursorRectIntersectsClipRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
